//--------------------------------------------------------------------  //088
//                                                                      //089
// Modul hcount                                                         //090
//                                                                      //091
// Timing der Zeilen; Steuersignale fuer Adressen, Bildpunkte,          //092
// horizontale Synchronisation und Timing des Bildes                    //093
//                                                                      //094
//--------------------------------------------------------------------  //095
//                                                                      //096
// Schnittstelle                                                        //097
//                                                                      //098
// NXTAD  im naechsten Takt Bildspeicherdaten uebernehmen,              //099
//        zur naechsten Bildspeicheradresse weiterschalten (1-aktiv)    //100
// NXTLN  im naechsten Takt Zeile hochzaehlen (1-aktiv)                 //101
// HSYNC  horizontale (Zeilen-)Synchronisation (1-aktiv)                //102
// CLOCK  Takt                                                          //103
// NRST   Initialisierung (0-aktiv)                                     //104
//                                                                      //105
//--------------------------------------------------------------------  //106
//                                                                      //107
// Timing und Arbeitsperiode von hcount:                                //108
//                                                                      //109
// `H1 Takte  linker Bildrand                                           //110
//            HSYNC=0, NXTAD=0, NTXLN=0                                 //111
// `H  Takte  Bildpunkte                                                //112
//            HSYNC=0, NXTLN=0, NXTAD wird alle 8 Takte gesetzt         //113
// `H2 Takte  rechter Bildrand                                          //114
//            HSYNC=0, NXTAD=0, NTXLN=0                                 //115
// `H3 Takte  horizontale Synchronisation                               //116
//            HSYNC=1, NXTAD=0, NXTLN wird im letzten Takt 1            //117
//                                                                      //118
//--------------------------------------------------------------------  //119
                                                                        //120
`include "discount_defs.v"

module hcount (NXTAD, NXTLN, HSYNC, CLOCK, NRST);                       //121
                                                                        //122
  // Ausgaenge                                                          //123
  output NXTAD,      // Bildadresse hochzaehlen                         //124
         NXTLN,      // Zeile hochzaehlen                               //125
         HSYNC;      // Zeilensynchronisation                           //126
                                                                        //127
  // Eingaenge                                                          //128
  input  CLOCK,      // Takt                                            //129
         NRST;       // Initialisierung (0-aktiv)                       //130
                                                                        //131
  // Port-Typen                                                         //132
  reg    NXTAD,                                                         //133
         NXTLN,                                                         //134
         HSYNC;                                                         //135
  wire   CLOCK,                                                         //136
         NRST;                                                          //137
                                                                        //138
  // interne Variablen                                                  //139
  reg [9:0] HCNT;    // Taktzaehler innerhalb einer Zeile               //140
  reg [1:0] HPHASE;  // Phasen horizontal:                              //141
                     //   0 linker Bildrand                             //142
                     //   1 Bildpunkte                                  //143
                     //   2 rechter Bildrand                            //144
                     //   3 horizontale Synchronisation                 //145
                                                                        //146
  //                                                                    //147
  // Taktzaehler aktualisieren                                          //148
  // (Periode `Hline, Bereich 0..`Hline-1)                              //149
  //                                                                    //150
  always @(posedge CLOCK or negedge NRST) begin                         //151
    if (NRST == 0)          HCNT <= `Hline-1;    // Reset               //152
    else                                                                //153
      if (HCNT == `Hline-1) HCNT <= 0;           // neue Zeile          //154
      else                  HCNT <= HCNT+1;                             //155
  end                                                                   //156
                                                                        //157
  //                                                                    //158
  // Phasenzaehler aktualisieren                                        //159
  //                                                                    //160
  always @(posedge CLOCK or negedge NRST) begin                         //161
    if (NRST == 0)  HPHASE <= 3;        // Reset                        //162
    else                                                                //163
      case (HCNT)                                                       //164
        0:          HPHASE <= 0;        // linker Bildrand              //165
        `H1:        HPHASE <= HPHASE+1; // Bildpunkte                   //166
        `H1+`H:     HPHASE <= HPHASE+1; // rechter Bildrand             //167
        `H1+`H+`H2: HPHASE <= HPHASE+1; // Bildsynchronisation          //168
      endcase                                                           //169
  end                                                                   //170
                                                                        //171
  //                                                                    //172
  // horizontale Synchronisation aktualisieren;                         //173
  // HSYNC wird ueber ein Register ausgegeben, um die Verzoegerung      //174
  // auf NXTAD von einem Takt zu kompensieren und so das Timing nach    //175
  // aussen einzuhalten                                                 //176
  //                                                                    //177
  always @(posedge CLOCK or negedge NRST) begin                         //178
    if (NRST == 0)     HSYNC <= 1;   // Reset                           //179
    else                                                                //180
      if (HPHASE == 3) HSYNC <= 1;                                      //181
      else             HSYNC <= 0;                                      //182
  end                                                                   //183
                                                                        //184
  //                                                                    //185
  // NXTAD und NXTLN aktualisieren                                      //186
  //                                                                    //187
  always @(HCNT or HPHASE) begin                                        //188
    if ((HCNT % `Bsz == 1 )                     // neues Byte           //189
      && (HPHASE == 1))               NXTAD = 1;                        //190
    else                              NXTAD = 0;                        //191
    if ((HCNT == 0) && (HPHASE == 3)) NXTLN = 1;                        //192
    else                              NXTLN = 0;                        //193
  end                                                                   //194
                                                                        //195
endmodule // hcount                                                     //196
